<div class="wbwriter table-of-contents">
{% if not_working %}{{not_working|safe}}{% else %}
<h1>{{title}}</h1>
{% if custom_enumeration %}
<style>
.toc {
    list-style: none;
}
</style>
{% endif %}
{% include "wbwriter/templatetags/table_of_contents_articles.html" %}{% endif %}
</div>
