# Generated by Django 5.0.12 on 2025-03-03 10:28

import modeltrans.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wbwriter', '0011_auto_20240103_0953'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='i18n',
            field=modeltrans.fields.TranslationField(fields=['title', 'content', 'slug'], required_languages=(), virtual_fields=True),
        ),
        migrations.AddField(
            model_name='publication',
            name='i18n',
            field=modeltrans.fields.TranslationField(fields=['title', 'slug', 'content'], required_languages=(), virtual_fields=True),
        ),
    ]
