# Generated by Django 4.2.6 on 2023-10-26 13:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        (
            "wbwriter",
            "0001_initial_squashed_squashed_0008_alter_article_author_alter_article_feedback_contact_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="DependantArticle",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "article",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="dependant_article_connections",
                        to="wbwriter.article",
                    ),
                ),
                (
                    "dependant_article",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="used_article_connections",
                        to="wbwriter.article",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
